#!/usr/bin/perl -w
# Mark Kamichoff <prox@prolixium.com>
# Listens on a TCP port for connections from nc/MRTG
# Provides data for:
#	smartmontools: "ht,$devIN,$devOUT"
# 	lm-sensors: "ls,$chipIN,$sensorIN,$chipOUT,$sensorOUT"
#	apcaccess: "apcpercent" or "apcvoltage"
#	acpi thermal zones: "at"
#	sectors R/W: "ds,$node"
# Version 1.5 [20061231]

use POSIX qw(setsid);
use IO::Socket;
use Getopt::Std;

# Change these, if you need to
$sensors="/usr/bin/sensors";
$smartctl="/usr/sbin/smartctl";
$apcaccess="/usr/sbin/apcaccess";

# Displays syntax and exists
sub syntax {
	print "Syntax: mrtg-rmt [-dhp port]\n";
	exit(1);
}

# Daemonize subroutine
sub daemonize {
	# Taken from http://www.webreference.com/perl/tutorial/9/2.html
	# (I'm lazy)

	chdir '/'                 or die "Can't chdir to /: $!";
	umask 0;
	open STDIN, '/dev/null'   or die "Can't read /dev/null: $!";
	#open STDOUT, '>/dev/null' or die "Can't write to /dev/null: $!";
	open STDERR, '>/dev/null' or die "Can't write to /dev/null: $!";
	defined(my $pid = fork)   or die "Can't fork: $!";
	exit if $pid;
	setsid                    or die "Can't start a new session: $!";
}

# Detault values
$daemon=0;
$port=249;

# Check arguments
%options=();
if(!getopts("p:d",\%options)) {
	syntax();
}
syntax() if defined $options{'h'};
$daemon=1 if defined $options{'d'};
if(defined $options{'p'}) {
	if($options{'p'} gt 0 && $options{'p'} lt 65536) {
		$port = $options{'p'};
	} else {
		print "Error: Port number $options{'p'} is out of range.\n";
		syntax();
	}
}

# Create socket
my $sock = new IO::Socket::INET (
		LocalHost => '0.0.0.0',
		LocalPort => $port,
		Proto => 'tcp',
		Listen => 1,
		Reuse => 1,
		);
die "Could not create socket: $!\n" unless $sock;

while(1) {
	my $new_sock = $sock->accept();
	if(!$daemon) {
		print "Connection from: ".inet_ntoa ((sockaddr_in($new_sock->peername))[1])."\n";
	}
	$recv = <$new_sock>;

	# If nothing
	if(!$recv) {
		next;
	}

	# hdd Temperature_Celsius (ht,devIN,devOUT)
	if($recv =~ m/^ht,(\/dev\/[^,]+),(\/dev\/.+)$/) {
		$devIN=$1;

		# this is SO STUPID
		$devOUT=$2;
		if($devOUT =~ m/\r$/) {
			chop $devOUT;
		}

		# Input
		@oarrIN = split(/\n/,`$smartctl -a $devIN`);
		$found=0;
		foreach $line (@oarrIN) {
			if($line =~ m/Temperature_Celsius\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+(\d+)/) {
				print $new_sock "$1\n";
				$found=1;
				last;
			}
		}
		if(!$found) { print $new_sock "0\n"; }

		# Output
		@oarrOUT = split(/\n/,`$smartctl -a $devOUT`);
		$found=0;
		foreach $line (@oarrOUT) {
			if($line =~ m/Temperature_Celsius\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+(\d+)/) {
				print $new_sock "$1\n";
				$found=1;
				last;
			}
		}
		if(!$found) { print $new_sock "0\n"; }

		# Normal stuff
		print $new_sock `uptime`;
		print $new_sock `hostname -f`;
	# lm sensors (ls,chipIN,sensorIN,chipOUT,sensorOUT)
	} elsif ($recv =~ m/^ls,([^,]+),([^,]+),([^,]+),(.+)$/) {
		$chipIN=$1;
		$sensorIN=$2;
		$chipOUT=$3;

		# this is SO STUPID
		$sensorOUT=$4;
		if($sensorOUT =~ m/\r$/) {
			chop $sensorOUT;
		}

		# Input - since we print that first
		@oarrIN = split(/\n/,`$sensors $chipIN`);
		$found=0;
		foreach $line (@oarrIN) {
			if($line =~ m/^$sensorIN\:[^\d]+(\d+)/) {
				print $new_sock "$1\n";
				$found=1;
				last;
			}
		}
		if(!$found) { print $new_sock "0\n"; }

		# Output - since we print that first
		@oarrOUT = split(/\n/,`$sensors $chipOUT`);
		$found=0;
		foreach $line (@oarrOUT) {
			if($line =~ m/^$sensorOUT\:[^\d]+(\d+)/) {
				print $new_sock "$1\n";
				$found=1;
				last;
			}
		}
		if(!$found) { print $new_sock "0\n"; }

		# Normal stuff
		print $new_sock `uptime`;
		print $new_sock `hostname -f`;

	# apcaccess percentage (in: load, out: batchrg)
	} elsif ($recv =~ m/^apcpercent$/) {

		# input (load percentage)
		@oarrIN = split(/\n/, `$apcaccess`);
		$found=0;
		foreach $line (@oarrIN) {
			if($line =~ m/^LOADPCT\s*:\s*(\d+)\./) {
				print $new_sock "$1\n";
				$found=1;
				last;
			}
		}
		if(!$found) { print $new_sock "0\n"; }

		# output (charge percentage)
		@oarrIN = split(/\n/, `$apcaccess`);
		$found=0;
		foreach $line (@oarrIN) {
			if($line =~ m/^BCHARGE\s*:\s*(\d+)\./) {
				print $new_sock "$1\n";
				$found=1;
				last;
			}
		}
		if(!$found) { print $new_sock "0\n"; }

		# Normal stuff
		print $new_sock `uptime`;
		print $new_sock `hostname -f`;

	# apcaccess voltage (in: bat, out: line)
	} elsif ($recv =~ m/^apcvoltage$/) {

		# input (battery output voltage)
		@oarrIN = split(/\n/, `$apcaccess`);
		$found=0;
		foreach $line (@oarrIN) {
			if($line =~ m/^BATTV\s*:\s*(\d+)\./) {
				print $new_sock "$1\n";
				$found=1;
				last;
			}
		}
		if(!$found) { print $new_sock "0\n"; }

		# output (line voltage)
		@oarrIN = split(/\n/, `$apcaccess`);
		$found=0;
		foreach $line (@oarrIN) {
			if($line =~ m/^LINEV\s*:\s*(\d+)\./) {
				print $new_sock "$1\n";
				$found=1;
				last;
			}
		}
		if(!$found) { print $new_sock "0\n"; }

		# Normal stuff
		print $new_sock `uptime`;
		print $new_sock `hostname -f`;

	# ACPI thermal zone (at)
	# If you have one CPU, shows the temp as in/out
	# If you have 2, shows in as cpu0, and out as cpu1
	# If you have 2+, sorry, will only show the first two :)
	} elsif ($recv =~ m/^at$/) {
		# multiple cores
		if(-e "/proc/acpi/thermal_zone/THM0"
				&& -e "/proc/acpi/thermal_zone/THM1") {

			$cpu0 = `cat /proc/acpi/thermal_zone/THM0/temperature`;
			$cpu1 = `cat /proc/acpi/thermal_zone/THM1/temperature`;
			chomp $cpu0;
			chomp $cpu1;

			# cpu0
			if($cpu0 =~ m/^temperature:\s+(\d+)\sC/) {
				print $new_sock "$1\n";
			} else {
				print $new_sock "0\n";
			}

			# cpu1
			if($cpu1 =~ m/^temperature:\s+(\d+)\sC/) {
				print $new_sock "$1\n";
			} else {
				print $new_sock "0\n";
			}

		} elsif(-e "/proc/acpi/thermal_zone/THM0"
				|| -e "/proc/acpi/thermal_zone/THRM") {
			if(-e "/proc/acpi/thermal_zone/THM0/temperature") {
				$cpu0 = `cat /proc/acpi/thermal_zone/THM0/temperature`;
			} else {
				$cpu0 = `cat /proc/acpi/thermal_zone/THRM/temperature`;
			}
			chomp $cpu0;

			if($cpu0 =~ m/^temperature:\s+(\d+)\sC/) {
				print $new_sock "$1\n$1\n";
			} else {
				print $new_sock "0\n0\n";
			}
			
		} else {
			print $new_sock "0\n0\n";
		}


		# Normal stuff
		print $new_sock `uptime`;
		print $new_sock `hostname -f`;
	
	# disk sectors read/written
	# specify block device (ds,hda)
	} elsif($recv =~ m/^ds,([a-z0-9]+)$/) {
		if(-e "/sys/block/$1/stat") {
			$out = `cat /sys/block/$1/stat`;
			chomp $out;
			if($out =~ m/^\s*\d+\s+\d+\s+(\d+)\s+\d+\s+\d+\s+\d+\s+(\d+)\s+\d+\s+\d+\s+\d+\s+\d+$/) {
				print $new_sock "$1\n$2\n";
			} else {
				print $new_sock "0\n0\n";
			}
		} else {
			print $new_sock "0\n0\n";
		}

		# Normal stuff
		print $new_sock `uptime`;
		print $new_sock `hostname -f`;
	} else {
		print $new_sock "0\n0\n";
		print $new_sock "An error has occurred.\n";
		print $new_sock `hostname -f`;
	}

	# Print a bye-bye message
	if(!$daemon) {
		print "Connection to ".inet_ntoa ((sockaddr_in($new_sock->peername))[1])." has been terminated.\n";
	}
}

# We'll never get here, since we don't fork!
close($sock);
